<?php

namespace Drupal\group_entity_reference_filter\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Entity\Attribute\EntityReferenceSelection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Entity reference selection plugin that filters by current group context.
 */
#[EntityReferenceSelection(
  id: "group_filter",
  label: new TranslatableMarkup("Group membership filter"),
  group: "group_filter",
  weight: 0
)]
class GroupFilterSelection extends DefaultSelection {

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected RouteMatchInterface $routeMatch;

  /**
   * The group relation type manager.
   *
   * @var \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface
   */
  protected GroupRelationTypeManagerInterface $groupRelationTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleHandlerInterface $module_handler,
    AccountInterface $current_user,
    EntityFieldManagerInterface $entity_field_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    EntityRepositoryInterface $entity_repository,
    RouteMatchInterface $route_match,
    GroupRelationTypeManagerInterface $group_relation_type_manager
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $entity_type_manager,
      $module_handler,
      $current_user,
      $entity_field_manager,
      $entity_type_bundle_info,
      $entity_repository
    );

    $this->routeMatch = $route_match;
    $this->groupRelationTypeManager = $group_relation_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('current_user'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity.repository'),
      $container->get('current_route_match'),
      $container->get('group_relation_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS'): QueryInterface {
    $query = parent::buildEntityQuery($match, $match_operator);
    $this->addGroupFilter($query);
    return $query;
  }

  /**
   * Gets the group relationship entity type ID.
   *
   * @return string
   *   The entity type ID.
   */
  protected function getGroupRelationshipTypeId(): string {
    return $this->entityTypeManager->hasDefinition('group_relationship')
      ? 'group_relationship'
      : 'group_content';
  }

  /**
   * Gets the group IDs for the current context.
   *
   * @return array
   *   Array of group IDs.
   */
  protected function getGroupIds(): array {
    $entity = $this->getCurrentEntity();
    if (!$entity) {
      return [];
    }

    // If entity is a group, use its ID
    if ($entity->getEntityTypeId() === 'group') {
      return [$entity->id()];
    }

    // Get relationships for existing entity
    $relationship_type = $this->getGroupRelationshipTypeId();
    $relationships = $this->entityTypeManager
      ->getStorage($relationship_type)
      ->loadByProperties(['entity_id' => $entity->id()]);

    $group_ids = [];
    foreach ($relationships as $relationship) {
      $group_ids[] = $relationship->gid->target_id;
    }

    return array_unique($group_ids);
  }

  /**
   * Adds group filtering to the query.
   *
   * @param \Drupal\Core\Entity\Query\QueryInterface $query
   *   The entity query to modify.
   */
  protected function addGroupFilter(QueryInterface $query): void {
    // Check if group module is enabled
    if (!$this->moduleHandler->moduleExists('group')) {
      return;
    }

    $group_ids = $this->getGroupIds();
    if (empty($group_ids)) {
      return;
    }

    $target_type = $this->configuration['target_type'];
    $entity_type = $this->entityTypeManager->getDefinition($target_type);
    $id_key = $entity_type->getKey('id');

    // Handle user memberships differently from other entity types
    $relationship_type = $this->getGroupRelationshipTypeId();
    $relationship_query = $this->entityTypeManager->getStorage($relationship_type)->getQuery()
      ->condition('gid', $group_ids, 'IN')
      ->accessCheck(FALSE);

    if ($target_type === 'user') {
      $relationship_query->condition('plugin_id', 'group_membership');
    }
    else {
      // Get all plugin IDs for the target entity type
      $plugin_ids = $this->groupRelationTypeManager->getPluginIdsByEntityTypeId($target_type);
      if (empty($plugin_ids)) {
        return;
      }
      $relationship_query->condition('plugin_id', $plugin_ids, 'IN');
    }

    // Execute and get related entities
    $entity_ids = $relationship_query->execute();
    if (!empty($entity_ids)) {
      // Load the relationships to get the actual entity IDs
      $relationships = $this->entityTypeManager->getStorage($relationship_type)->loadMultiple($entity_ids);
      $target_entity_ids = [];
      foreach ($relationships as $rel) {
        if ($entity = $rel->getEntity()) {
          $target_entity_ids[] = $entity->id();
        }
      }

      if (!empty($target_entity_ids)) {
        $query->condition($id_key, $target_entity_ids, 'IN');
      }
    }
  }

  /**
   * Gets the current entity being edited.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The current entity or NULL if not found.
   */
  protected function getCurrentEntity(): ?EntityInterface {
    foreach ($this->routeMatch->getParameters() as $parameter) {
      if ($parameter instanceof EntityInterface) {
        return $parameter;
      }
    }
    return NULL;
  }

}
